#!/usr/bin/python

#      GTKNetTraf, a client for the NetTraf traffic monitor.
#      Copyright (C) 2005  Michael Sheldon (mts4 at aber dot ac dot uk)       
#                                                                              
#      This program is free software; you can redistribute it and/or           
#      modify it under the terms of the GNU General Public License             
#      as published by the Free Software Foundation; either version 2       
#      of the License, or (at your option) any later version.               
#                                                                           
#      This program is distributed in the hope that it will be useful,      
#      but WITHOUT ANY WARRANTY; without even the implied warranty of       
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
#      GNU General Public License for more details.                         
#                                                                           
#      You should have received a copy of the GNU General Public License    
#      along with this program; if not, write to the Free Software          
#      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,           
#      MA  02110-1301, USA                                                  
		  

import gtk, gtk.glade, gobject, string, select, sys, os, time
from socket import *
from threading import Thread

class Network(Thread):

	def __init__(self, parent, host, port, interface, machine):
		Thread.__init__(self)
		gtk.threads_enter()
		self.running = 1
		self.parent = parent
		self.machine = machine
		try:
			self.sock = socket(AF_INET, SOCK_STREAM)
        	        self.sock.connect((host, port))
			self.sock.send("ITF " + interface)
			self.sock.setblocking(0) #Non-blocking
			self.fsock = self.sock.makefile("rb")
		except:
			dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Could not connect to the server \"" + host + "\"; please check your settings.")
			dlg.run()
			dlg.destroy()
			self.parent.showPrefs(None)
			self.sock = None
			self.running = 0
		

	def shutdown(self):
		self.running = 0

	def run(self):
		while self.running:
			gtk.threads_enter()
			r, w, e = select.select([self.sock], [], [], 0)
			if r:
				data = self.fsock.readline()
				if data[:4] == "DATA":
					inData, outData = string.split(data[5:], " ")
					self.parent.update(inData, outData, self.machine)
				if data[:5] == "ERROR":
					dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Received an error from the server; are you sure you have a valid network interface selected?")
					dlg.run()
					dlg.destroy()
					self.sock.send("QUIT")
					self.sock.close()
					self.sock = None
					self.running = 0
			gtk.threads_leave()
			time.sleep(0.02)
		if self.sock:
			self.sock.setblocking(1)
			self.sock.send("QUIT")
			self.sock.close()
			

class GTKNetTraf(gtk.Window):
	
	def __init__(self):
		self.mainGlade = gtk.glade.XML("gtknettraf.glade", "mainWin", None)
		self.vbox2 = self.mainGlade.get_widget("vbox2")
		self.ntbMain = gtk.Notebook()
		self.ntbMain.set_show_border(0)
		self.vbox2.add(self.ntbMain)
		self.tblMain = self.mainGlade.get_widget("tblMain")
		self.oldIn = {}
		self.oldOut = {}
		self.inList = {}
		self.outList = {}
		self.showingGraph = 0
		
		self.loadPrefs()
		
		pages = 0
		self.lblInSpeed = {}
		self.lblOutSpeed = {}
		self.barInSpeed = {}
		self.barOutSpeed = {}
		self.lblTotalRecv = {}
		self.lblTotalSent = {}
		self.lblUnits = {}
		self.lblUnits2 = {}
		self.networks = {}
		for i in range(1, 11):
			num = str(i)
			if self.prefs.has_key('enabled' + num) and self.prefs['enabled' + num] == 1:
				pages += 1
				label = gtk.Label(self.prefs['host' + num])
				table = gtk.Table(4, 3)
				table.attach(gtk.Label("In: "), 0, 1, 0, 1, gtk.FILL, 0)
				table.attach(gtk.Label("Out: "), 0, 1, 1, 2, gtk.FILL, 0)
				table.attach(gtk.Label("Total Received: "), 0, 1, 2, 3, gtk.FILL, 0)
				self.lblInSpeed[i] = gtk.Label("0")
				self.lblInSpeed[i].set_size_request(107, -1)
				self.lblOutSpeed[i] = gtk.Label("0")
				self.lblTotalRecv[i] = gtk.Label("0")
				table.attach(self.lblInSpeed[i], 1, 2, 0, 1, gtk.FILL, 0)
				table.attach(self.lblOutSpeed[i], 1, 2, 1, 2, gtk.FILL, 0)
				table.attach(self.lblTotalRecv[i], 1, 2, 2, 3, gtk.FILL, 0)
				units = self.prefs['units' + num] + "/second  "
				self.lblUnits[i] = gtk.Label(units)
				self.lblUnits2[i] = gtk.Label(units)
				table.attach(self.lblUnits[i], 2, 3, 0, 1, gtk.FILL, 0)
				table.attach(self.lblUnits2[i], 2, 3, 1, 2, gtk.FILL, 0)
				table.attach(gtk.Label("Total Sent: "), 2, 3, 2, 3, gtk.FILL, 0)
				self.barInSpeed[i] = gtk.ProgressBar()
				self.barOutSpeed[i] = gtk.ProgressBar()
				self.lblTotalSent[i] = gtk.Label("0")
				table.attach(self.barInSpeed[i], 3, 4, 0, 1, gtk.FILL, 0)
				table.attach(self.barOutSpeed[i], 3, 4, 1, 2, gtk.FILL, 0)
				table.attach(self.lblTotalSent[i], 3, 4, 2, 3, gtk.FILL, 0)
				table.set_row_spacings(3)
				self.ntbMain.append_page(table, label)
				self.inList[i] = []
				self.outList[i] = []
				for n in range(0, 40):
					self.inList[i].append(0)
					self.outList[i].append(0)
				self.networks[i] = Network(self, self.prefs['host' + num], int(self.prefs['port' + num]), self.prefs['interface' + num], i)
				self.networks[i].start()
						 

		self.ntbMain.show_all()
		#Hide tabs if we only have one machine
		if pages <= 1:
			self.ntbMain.set_show_tabs(0)

		signals = {
			"on_mainWin_destroy" : self.quit, 
			"on_quit1_activate" : self.quit,
			"on_graph1_activate" : self.showGraph,
			"on_about1_activate" : self.showAbout,
			"on_preferences1_activate" : self.showPrefs,
		}

		self.mainGlade.signal_autoconnect(signals)
	
	def quit(self, widget):
		gtk.main_quit()
		for machine in self.networks:
			network = self.networks.get(machine)
			network.shutdown()

	def loadPrefs(self):
		home = os.path.expanduser("~")
		filename = os.path.join(home, ".gtknettraf")
		try:
			file = open(filename, 'r')
			keys = []
			values = []
			for line in file.readlines():
				line = line.replace("\n", "")
				line = line.lower()
				key, value = string.split(line, "=")
				if key[:7] == "enabled":
					if value == "true": value = 1
					elif value == "false": value = 0
					else: value = int(value)
				#Compatibility for old configs
				if key == "host": key = "host1"
				if key == "port": key = "port1"
				if key == "interface": key = "interface1"
				if key == "units": key = "units1"
				if key == "enabled": key = "enabled1"
				keys.append(key)
				values.append(value)
			self.prefs = dict(zip(keys, values))
			file.close()
		except:
			print "Config file missing/corrupt; recreating."
			try:
				file = open(filename, 'w')
				defaults = "enabled=1\nhost=localhost\nport=4664\ninterface=eth0\nunits=Bytes\n"
				file.write(defaults)
				file.close()
				self.loadPrefs()
			except:
				print "Error writing new config file."
				sys.exit(1) #Safe not to use self.quit() here, since network uninitialised

	def showAbout(self, widget):
		aboutGlade = gtk.glade.XML("gtknettraf.glade", "dlgAbout", None)

	def showGraph(self, widget):
		self.graphGlade = gtk.glade.XML("gtknettraf.glade", "graphWin", None)
		self.ntbGraph = gtk.Notebook()
		self.ntbGraph.set_show_border(0)
		self.graphGlade.get_widget("graphWin").add(self.ntbGraph)
		self.drawGraph = {}
		pages = 0
		for i in range(1, 11):
			num = str(i)
			if self.prefs.has_key('enabled' + num) and self.prefs['enabled' + num] == 1:
				pages += 1
				label = gtk.Label(self.prefs['host' + num])
				self.drawGraph[i] = gtk.DrawingArea()
				self.drawGraph[i].set_size_request(390, 246)
				self.ntbGraph.append_page(self.drawGraph[i], label)
				self.drawGraph[i].realize()
				self.drawGraph[i].connect("expose_event", self.exposeGraph, None)
		self.ntbGraph.show_all()
		
		#Hide tabs if we only have one machine
		if pages <= 1:
			self.ntbGraph.set_show_tabs(0)
							
		
		self.showingGraph = 1
	
		signals = {
			"on_graphWin_destroy" : self.closeGraph,
		}

		self.graphGlade.signal_autoconnect(signals)
		
		self.yellow_gc = gtk.gdk.GC(self.drawGraph[1].window)
		self.yellow_gc.set_rgb_fg_color(gtk.gdk.color_parse('yellow'))
		self.red_gc = gtk.gdk.GC(self.drawGraph[1].window)
		self.red_gc.set_rgb_fg_color(gtk.gdk.color_parse('red'))
		self.black_gc = gtk.gdk.GC(self.drawGraph[1].window)
		self.black_gc.set_rgb_fg_color(gtk.gdk.color_parse('black'))

	def showPrefs(self, widget):
		self.prefsGlade = gtk.glade.XML("gtknettraf.glade", "prefsWin", None)
		self.ckbEnabled = self.prefsGlade.get_widget("ckbEnabled")
		self.spnMachine = self.prefsGlade.get_widget("spnMachine")
		self.txtHost = self.prefsGlade.get_widget("txtHost")
		self.txtPort = self.prefsGlade.get_widget("txtPort")
		self.txtInterface = self.prefsGlade.get_widget("txtInterface")
		self.cmbUnits = self.prefsGlade.get_widget("cmbUnits")
		self.txtHost.set_text(self.prefs['host1'])
		self.txtPort.set_text(self.prefs['port1'])
		self.txtInterface.set_text(self.prefs['interface1'])
		units = self.prefs['units1']

		if units == "kilobytes":
			self.cmbUnits.set_active(1)
		elif units == "megabytes":
			self.cmbUnits.set_active(2)
		else:
			self.cmbUnits.set_active(0)

		#Backward compatibility for old config
		try:
			if self.prefs['enabled1'] == 1:
				self.ckbEnabled.set_active(1)
		except:
			self.prefs['enabled1'] = 1
			self.ckbEnabled.set_active(1)

		self.enableToggle(None) #Toggle first machine

		signals = {
			"on_spnMachine_value_changed" : self.changeMachine,
			"on_ckbEnabled_toggled" : self.enableToggle,
			"on_txtHost_changed" : self.updatePrefs,
			"on_txtPort_changed" : self.updatePrefs,
			"on_txtInterface_changed" : self.updatePrefs,
			"on_cmbUnits_changed" : self.updatePrefs,
			"on_btnSave_clicked" : self.savePrefs,
			"on_btnCancel_clicked" : self.closePrefs,
		}

		self.prefsGlade.signal_autoconnect(signals)
		
	def savePrefs(self, widget):
		home = os.path.expanduser("~")
		file = open(home + "/.gtknettraf", 'w')
		prefstr = ''
		for key in self.prefs:
			value = self.prefs.get(key)
			prefstr += key + "=" + str(value) + "\n"
		file.write(prefstr)
		file.close()
		self.closePrefs(widget)

	def closePrefs(self, widget):
		prefsWin = self.prefsGlade.get_widget("prefsWin")
		prefsWin.destroy()

	def changeMachine(self, widget):
		machine = str(int(self.spnMachine.get_value()))
		try:
			if self.prefs['enabled' + machine] == 1:
				self.ckbEnabled.set_active(1)
			else:
				self.ckbEnabled.set_active(0)
		except:
			self.ckbEnabled.set_active(0)
		try:
			self.txtHost.set_text(self.prefs['host' + machine])
		except:
			self.txtHost.set_text("")
		try:
			self.txtPort.set_text(self.prefs['port' + machine])
		except:
			self.txtPort.set_text("")
		try:
			self.txtInterface.set_text(self.prefs['interface' + machine])
		except:
			self.txtInterface.set_text("")
		try:
			if self.prefs['units' + machine] == "kilobytes":
				self.cmbUnits.set_active(1)
			elif self.prefs['units' + machine] == "megabytes":
				self.cmbUnits.set_active(2)
			else:
				self.cmbUnits.set_active(0)
		except:
			self.cmbUnits.set_active(0)

	#Enable/Disable entry fields
	def enableToggle(self, widget):
		value = self.ckbEnabled.get_active()
		machine = str(int(self.spnMachine.get_value()))
		self.prefs['enabled' + machine] = value
		self.txtHost.set_sensitive(value)
		self.txtPort.set_sensitive(value)
		self.txtInterface.set_sensitive(value)
		self.cmbUnits.set_sensitive(value)

	#This is kind of inefficient, needs changing
	def updatePrefs(self, widget):
		if self.ckbEnabled.get_active(): #Make sure we're not just cycling through disabled machines
			machine = str(int(self.spnMachine.get_value()))
			if widget == self.txtHost:
				self.prefs['host' + machine] = self.txtHost.get_text()
			elif widget == self.txtPort:
				self.prefs['port' + machine] = self.txtPort.get_text()
			elif widget == self.txtInterface:
				self.prefs['interface' + machine] = self.txtInterface.get_text()
			elif widget == self.cmbUnits:
				units = self.cmbUnits.get_active_text().lower()
				self.prefs['units' + machine] = units
				#Update current interface
				try:
					self.lblUnits[int(machine)].set_text(units + "/seconds  ")
					self.lblUnits2[int(machine)].set_text(units + "/seconds  ")
				except:
					pass #Error supression; failure just indicates a machine isn't enabled, can just be ignored

	def closeGraph(self, widget):
		self.showingGraph = 0

	def exposeGraph(self, widget, *args):
		event = args[0]
	        top, left, width, height = event.area
		self.updateGraph(top, left, width, height, self.ntbGraph.get_current_page() + 1)
	

	def updateGraph(self, top, left, width, height, machine):
	        self.drawGraph[machine].window.draw_rectangle(self.black_gc,  1, top, left, width, height)
		for i in range(1, 40):
			width, height = self.drawGraph[machine].window.get_size()
                	x1 = (i - 1) * (width / 39)
			x2 = i * (width / 39)
			modifier = 100
			if self.prefs['units' + str(machine)] == "kilobytes":
				modifier = 0.5
			if self.prefs['units' + str(machine)] == "megabytes":
				modifier = 0.05
			iy1 = height - self.inList[machine][i - 1] / modifier
			iy2 = height - self.inList[machine][i] / modifier
			oy1 = height - self.outList[machine][i - 1] / modifier
			oy2 = height - self.outList[machine][i] / modifier
			self.drawGraph[machine].window.draw_line(self.yellow_gc, int(x1), int(iy1), int(x2), int(iy2))
			self.drawGraph[machine].window.draw_line(self.red_gc, int(x1), int(oy1), int(x2), int(oy2))
																												
	

	def update(self, inData, outData, machine):
		if self.prefs['units' + str(machine)] == "kilobytes":
			self.lblTotalSent[machine].set_text(str(round(int(outData) / 1024.0, 2)))
			self.lblTotalRecv[machine].set_text(str(round(int(inData) / 1024.0, 2)))
		elif self.prefs['units' + str(machine)] == "megabytes":
			self.lblTotalSent[machine].set_text(str(round(int(outData) / (1024.0 * 1024.0), 2)))
			self.lblTotalRecv[machine].set_text(str(round(int(inData) / (1024.0 * 1024.0), 2)))
		else:
			self.lblTotalSent[machine].set_text(str(int(outData)))
			self.lblTotalRecv[machine].set_text(str(int(inData)))
		if self.oldIn.has_key(machine):
			inSpeed = int(inData) - int(self.oldIn[machine])
			outSpeed = int(outData) - int(self.oldOut[machine])
			if self.prefs['units' + str(machine)] == "kilobytes":
				inSpeed = round(inSpeed / 1024.0, 2)
				outSpeed = round(outSpeed / 1024.0, 2)
			if self.prefs['units' + str(machine)] == "megabytes":
				inSpeed = round(inSpeed / (1024 * 1024.0), 2)
				outSpeed = round(outSpeed / (1024 * 1024.0), 2)
			self.lblInSpeed[machine].set_text(str(inSpeed))
			self.lblOutSpeed[machine].set_text(str(outSpeed))
			modifier = 20000.0
			if self.prefs['units' + str(machine)] == "kilobytes":
				modifier = 100.0
			if self.prefs['units' + str(machine)] == "megabytes":
				modifier = 10.0
			inPercent = inSpeed/modifier
			outPercent = outSpeed/modifier
			if inPercent > 1: inPercent = 1
			if outPercent > 1: outPercent = 1
			self.barInSpeed[machine].set_fraction(inPercent)
			self.barOutSpeed[machine].set_fraction(outPercent)
			for i in range(0, 39):
				self.inList[machine][i] = self.inList[machine][i + 1]
				self.outList[machine][i] = self.outList[machine][i + 1]
			self.inList[machine][39] = inSpeed
			self.outList[machine][39] = outSpeed
			if self.showingGraph:
				width, height = self.drawGraph[machine].window.get_size()
				self.updateGraph(0, 0, width, height, machine)
		self.oldIn[machine] = inData
		self.oldOut[machine] = outData


app = GTKNetTraf()
gtk.threads_init()
gtk.main()
